@@ -8,12 +8,17 @@ import asyncio
 import json
 import logging
 from datetime import datetime, timedelta, timezone
-from typing import Any, Optional
+from typing import Any, Awaitable, Callable, Optional
 
 import aiohttp
 
 from asusrouter.connection import Connection
-from asusrouter.const import DEFAULT_CACHE_TIME, DEFAULT_TIMEOUT
+from asusrouter.const import (
+    DEFAULT_CACHE_TIME,
+    DEFAULT_RESULT_SUCCESS,
+    DEFAULT_TIMEOUT,
+    RequestType,
+)
 from asusrouter.error import (
     AsusRouter404Error,
     AsusRouterAccessError,
@@ -21,32 +26,49 @@ from asusrouter.error import (
     AsusRouterDataError,
 )
 from asusrouter.modules.attributes import AsusRouterAttribute
-from asusrouter.modules.client import process_client
 from asusrouter.modules.data import AsusData, AsusDataState
 from asusrouter.modules.data_finder import (
     ASUSDATA_ENDPOINT_APPEND,
     ASUSDATA_MAP,
+    ASUSDATA_NVRAM,
     AsusDataFinder,
     AsusDataMerge,
+    add_conditional_data_alias,
+    add_conditional_data_rule,
+    remove_data_rule,
+)
+from asusrouter.modules.data_transform import (
+    transform_clients,
+    transform_cpu,
+    transform_ethernet_ports,
+    transform_network,
+    transform_wan,
+)
+from asusrouter.modules.endpoint import (
+    ENDPOINT_FORCE_REQUEST,
+    Endpoint,
+    EndpointControl,
+    EndpointType,
+    process,
+    read,
 )
-from asusrouter.modules.data_transform import transform_clients, transform_network
-from asusrouter.modules.endpoint import Endpoint, EndpointControl, process, read
 from asusrouter.modules.endpoint.error import AccessError
+from asusrouter.modules.firmware import Firmware
 from asusrouter.modules.flags import Flag
 from asusrouter.modules.identity import AsusDevice, collect_identity
-from asusrouter.modules.parental_control import ParentalControlRule
 from asusrouter.modules.port_forwarding import PortForwardingRule
 from asusrouter.modules.service import async_call_service
 from asusrouter.modules.state import (
     AsusState,
+    add_conditional_state,
     get_datatype,
     keep_state,
     save_state,
     set_state,
 )
 from asusrouter.tools import legacy
-from asusrouter.tools.converters import safe_list
-from asusrouter.tools.readers import merge_dicts, readable_mac
+from asusrouter.tools.converters import get_enum_key_by_value, safe_list
+from asusrouter.tools.readers import merge_dicts
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -54,15 +76,6 @@ _LOGGER = logging.getLogger(__name__)
 class AsusRouter:
     """The interface class."""
 
-    _cache_time: float = DEFAULT_CACHE_TIME
-    _hostname: str
-
-    _identity: Optional[AsusDevice] = None
-
-    _state: dict[AsusData, AsusDataState] = {}
-
-    _flags: Flag = Flag()
-
     def __init__(
         self,
         hostname: str,
@@ -72,17 +85,28 @@ class AsusRouter:
         use_ssl: bool = False,
         cache_time: Optional[float] = None,
         session: Optional[aiohttp.ClientSession] = None,
+        dumpback: Optional[Callable[..., Awaitable[None]]] = None,
     ):  # pylint: disable=too-many-arguments
         """Initialize the interface."""
 
         _LOGGER.debug("Initializing a new interface to `%s`", hostname)
 
         # Set the cache time
-        if cache_time:
-            self._cache_time = cache_time
+        self._cache_time = cache_time or DEFAULT_CACHE_TIME
 
         # Set the host
-        self._hostname = hostname
+        self._hostname: str = hostname
+
+        # Set the device identity
+        self._identity: Optional[AsusDevice] = None
+        self._state: dict[AsusData, AsusDataState] = {}
+
+        # Set the flags
+        self._flags: Flag = Flag()
+        # Time for change to take effect before available to fetch
+        self._needed_time: Optional[int] = None
+        # ID from the last called service
+        self._last_id: Optional[int] = None
 
         # Create a connection
         self._connection = Connection(
@@ -92,6 +116,7 @@ class AsusRouter:
             port=port,
             use_ssl=use_ssl,
             session=session,
+            dumpback=dumpback,
         )
 
     # ---------------------------
@@ -151,9 +176,35 @@ class AsusRouter:
         if led_state and led_state.data:
             _LOGGER.debug("Restoring LED state")
             await keep_state(
-                self.async_run_service, led_state.data["state"], self._identity
+                callback=self.async_run_service,
+                states=led_state.data["state"],
+                identity=self._identity,
             )
 
+        # Reset the reboot flag
+        self._reset_flag("reboot")
+
+    def _reset_flag(self, flag: str) -> None:
+        """Reset a flag."""
+
+        _LOGGER.debug("Triggered method _reset_flag")
+
+        # Check that AsusData.FLAGS is available
+        if AsusData.FLAGS not in self._state:
+            return
+
+        # Get the data from the state
+        data = self._state[AsusData.FLAGS].data
+
+        # Check that data is a dict
+        if not isinstance(data, dict):
+            return
+
+        # Reset the flag
+        data.pop(flag, None)
+
+        _LOGGER.debug("Flag `%s` reset", flag)
+
     # ---------------------------
     # <-- Connection-related methods
     # ---------------------------
@@ -177,6 +228,62 @@ class AsusRouter:
             api_query=self.async_api_query,
         )
 
+        # Add conditional data rules
+        if self._identity:
+            firmware = self._identity.firmware
+            merlin = self._identity.merlin
+            fw_388 = Firmware(major="3.0.0.4", minor=388, build=0)
+            # Stock
+            if not merlin:
+                _LOGGER.debug("Adding conditional rules for stock firmware")
+                if fw_388 < firmware:
+                    add_conditional_state(
+                        AsusState.OPENVPN_CLIENT, AsusData.VPNC
+                    )
+                    add_conditional_state(
+                        AsusState.WIREGUARD_CLIENT, AsusData.VPNC
+                    )
+                    add_conditional_data_alias(
+                        AsusData.OPENVPN_CLIENT, AsusData.VPNC
+                    )
+                    add_conditional_data_alias(
+                        AsusData.WIREGUARD_CLIENT, AsusData.VPNC
+                    )
+                    add_conditional_data_rule(
+                        AsusData.OPENVPN_SERVER,
+                        AsusDataFinder(
+                            Endpoint.HOOK,
+                            nvram=ASUSDATA_NVRAM["openvpn_server_388"],
+                        ),
+                    )
+            # Merlin
+            else:
+                _LOGGER.debug("Adding conditional rules for Merlin firmware")
+                if fw_388 < firmware:
+                    add_conditional_data_rule(
+                        AsusData.VPNC,
+                        AsusDataFinder(
+                            Endpoint.HOOK,
+                            nvram=ASUSDATA_NVRAM["vpnc"],
+                        ),
+                    )
+            # Before 388
+            if firmware < fw_388:
+                # Remove VPNC rules
+                remove_data_rule(AsusData.VPNC)
+                remove_data_rule(AsusData.VPNC_CLIENTLIST)
+                # Remove WireGuard rules
+                remove_data_rule(AsusData.WIREGUARD)
+                remove_data_rule(AsusData.WIREGUARD_CLIENT)
+                remove_data_rule(AsusData.WIREGUARD_SERVER)
+
+            # Ookla Speedtest
+            if self._identity.ookla is False:
+                remove_data_rule(AsusData.SPEEDTEST)
+                # remove_data_rule(AsusData.SPEEDTEST_HISTORY)
+                remove_data_rule(AsusData.SPEEDTEST_RESULT)
+                # remove_data_rule(AsusData.SPEEDTEST_SERVERS)
+
         # Return new identity
         return self._identity
 
@@ -188,13 +295,15 @@ class AsusRouter:
     # Request-related methods -->
     # ---------------------------
 
-    def _get_attribute(self, attribute: Optional[AsusRouterAttribute]) -> Optional[Any]:
+    def _get_attribute(
+        self, attribute: Optional[AsusRouterAttribute]
+    ) -> Optional[Any]:
         """Get an attribute value."""
 
         if not attribute:
             return None
 
-        match (attribute):
+        match attribute:
             case AsusRouterAttribute.MAC:
                 if self._identity:
                     return self._identity.mac
@@ -217,35 +326,42 @@ class AsusRouter:
         ):
             flags = state_flags.data
 
-        print("FLAGS: ", flags)
-
         if flags.get("reboot", False) is True:
             _LOGGER.debug("Reboot flag is set")
             await self._async_handle_reboot()
 
     async def async_api_query(
-        self, endpoint: Endpoint | EndpointControl, payload: Optional[str] = None
+        self, endpoint: EndpointType, payload: Optional[str] = None
     ):
         """A wrapper for the connection query method."""
 
         if endpoint in ASUSDATA_ENDPOINT_APPEND:
             payload = payload or ""
             for key, attribute in ASUSDATA_ENDPOINT_APPEND[endpoint].items():
-                value = self._get_attribute(attribute)
+                if isinstance(attribute, AsusRouterAttribute):
+                    value = self._get_attribute(attribute)
+                else:
+                    value = attribute
                 if value:
                     payload += f"{key}={value};"
             # Remove trailing semicolon
             payload = payload[:-1]
 
-        _LOGGER.debug("Triggered method async_api_query: %s | %s", endpoint, payload)
+        _LOGGER.debug(
+            "Triggered method async_api_query: %s | %s", endpoint, payload
+        )
 
-        return await self._connection.async_query(endpoint, payload)
+        request_type = ENDPOINT_FORCE_REQUEST.get(endpoint, RequestType.POST)
+
+        return await self._connection.async_query(
+            endpoint, payload, request_type=request_type
+        )
 
     async def async_api_load(
         self,
-        endpoint: Endpoint | EndpointControl,
+        endpoint: EndpointType,
         request: str = "",
-        retry: bool = False,
+        retry: int = 0,
     ) -> dict[str, Any]:
         """Load API endpoint with optional request."""
 
@@ -261,8 +377,10 @@ class AsusRouter:
             # Check whether we are not connected
             args = ex.args
             if args[1] == AccessError.AUTHORIZATION:
-                # Reconnect
-                await self.async_connect()
+                # Mark the connection as dropped
+                await self._async_drop_connection()
+                # Wait before repeating the request
+                await asyncio.sleep(1 + retry * 3)
                 # Repeat request once more and see what happens
                 return await self.async_api_load(endpoint, request, True)
             # Otherwise just raise the exception
@@ -306,11 +424,13 @@ class AsusRouter:
     async def async_api_command(
         self,
         commands: Optional[dict[str, str]],
-        endpoint: Endpoint | EndpointControl = EndpointControl.COMMAND,
+        endpoint: EndpointType = EndpointControl.COMMAND,
     ):
         """Send a command to the device."""
 
-        _LOGGER.debug("Triggered method async_api_command: %s | %s", endpoint, commands)
+        _LOGGER.debug(
+            "Triggered method async_api_command: %s | %s", endpoint, commands
+        )
 
         return await self.async_api_load(
             endpoint=endpoint,
@@ -321,7 +441,9 @@ class AsusRouter:
     # <-- Request-related methods
     # ---------------------------
 
-    def _where_to_get_data(self, datatype: AsusData) -> Optional[AsusDataFinder]:
+    def _where_to_get_data(
+        self, datatype: AsusData
+    ) -> Optional[AsusDataFinder]:
         """Get the list of endpoints to get data from."""
 
         _LOGGER.debug("Triggered method _where_to_get_data")
@@ -333,13 +455,20 @@ class AsusRouter:
 
         # Get the map
         data_map = ASUSDATA_MAP.get(datatype)
-        if not data_map:
+        # Consider aliases
+        while isinstance(data_map, AsusData):
+            data_map = ASUSDATA_MAP.get(data_map)
+        # Check if we have a map
+        if not isinstance(data_map, AsusDataFinder):
+            _LOGGER.debug("No map found for %s", datatype)
             return None
 
         # Check if endpoints are available
         for endpoint in data_map.endpoint:
             # Check endpoint availability in identity
-            if self._identity.endpoints and self._identity.endpoints.get(endpoint) in (
+            if self._identity.endpoints and self._identity.endpoints.get(
+                endpoint
+            ) in (
                 False,
                 None,
             ):
@@ -350,14 +479,24 @@ class AsusRouter:
 
         return data_map
 
-    def _transform_data(self, datatype: AsusData, data: Any) -> Any:
+    def _transform_data(
+        self, datatype: AsusData, data: Any, **kwargs: Any
+    ) -> Any:
         """Transform data if needed."""
 
         _LOGGER.debug("Triggered method _transform_data for `%s`", datatype)
 
         if datatype == AsusData.CLIENTS:
             _LOGGER.debug("Transforming clients data")
-            return transform_clients(data, self._state.get(AsusData.CLIENTS))
+            return transform_clients(
+                data,
+                self._state.get(AsusData.CLIENTS),
+                aimesh=self._identity.aimesh if self._identity else False,
+            )
+
+        if datatype == AsusData.CPU:
+            _LOGGER.debug("Transforming CPU data")
+            return transform_cpu(data)
 
         if datatype == AsusData.NETWORK:
             _LOGGER.debug("Transforming network data")
@@ -365,10 +504,71 @@ class AsusRouter:
                 data,
                 self._identity.services if self._identity else [],
                 self._state.get(AsusData.NETWORK),
+                model=self._identity.model if self._identity else None,
+            )
+
+        if datatype == AsusData.PORTS:
+            _LOGGER.debug("Transforming port data")
+            return transform_ethernet_ports(
+                data,
+                self._identity.mac if self._identity else None,
+            )
+
+        if datatype == AsusData.WAN:
+            _LOGGER.debug("Transforming WAN data")
+            return transform_wan(
+                data,
+                self._identity.services if self._identity else [],
             )
 
         return data
 
+    def _drop_data(self, datatype: AsusData, endpoint: EndpointType) -> bool:
+        """Check whether data should be dropped. This is required
+        for some data obtained from multiple endpoints."""
+
+        if not self._identity:
+            return False
+
+        if datatype == AsusData.OPENVPN_CLIENT:
+            if self._identity.merlin is True:
+                return endpoint == Endpoint.HOOK
+
+        return False
+
+    async def _check_prerequisites(self, datatype: AsusData) -> None:
+        """Check prerequisites before fetching data."""
+
+        _LOGGER.debug(
+            "Triggered method _check_prerequisites for datatype `%s`", datatype
+        )
+
+        # A placeholder for future checks
+        return
+
+    async def _check_postrequisites(self, datatype: AsusData) -> None:
+        """Check postrequisites after fetching data.
+
+        This method is also used to fetch additional data."""
+
+        _LOGGER.debug(
+            "Triggered method _check_postrequisites for datatype `%s`",
+            datatype,
+        )
+
+        # Firmware
+        if datatype == AsusData.FIRMWARE:
+            # Check if update is available
+            firmware = self._state[AsusData.FIRMWARE].data
+            if firmware and firmware["state"] is True:
+                # Get release notes
+                release_note = await self.async_get_data(
+                    AsusData.FIRMWARE_NOTE, force=True
+                )
+                if release_note:
+                    firmware.update(release_note)
+        return
+
     def _check_state(self, datatype: Optional[AsusData]) -> None:
         """Make sure the state object is available."""
 
@@ -381,11 +581,43 @@ class AsusRouter:
         if datatype not in self._state:
             self._state[datatype] = AsusDataState(
                 timestamp=(
-                    datetime.now(timezone.utc) - timedelta(seconds=2 * self._cache_time)
+                    datetime.now(timezone.utc)
+                    - timedelta(seconds=2 * self._cache_time)
                 )
             )
 
-    async def async_get_data(self, datatype: AsusData, force: bool = False) -> Any:
+    def _return_state(self, datatype: AsusData, **kwargs: Any) -> Any:
+        """Return a proper state."""
+
+        _LOGGER.debug("Triggered method _return_state")
+
+        # Get the state
+        state = self._state[datatype].data
+
+        if datatype == AsusData.PORTS:
+            own_mac = self._identity.mac if self._identity else None
+
+            # Get the device selected
+            device = kwargs.get("device", None)
+
+            match device:
+                case None:
+                    if isinstance(state, dict):
+                        return state.get(own_mac, {})
+                    return state
+                case "all":
+                    return state
+                # Case when substate is a MAC address
+                case a if isinstance(a, str):
+                    if isinstance(state, dict) and a in state:
+                        return state[a]
+                    return {}
+
+        return state
+
+    async def async_get_data(
+        self, datatype: AsusData, force: bool = False, **kwargs: Any
+    ) -> Any:
         """Generic method to get data from the device."""
 
         # Check if we have a state object for this data
@@ -394,19 +626,24 @@ class AsusRouter:
         # If state object is active, wait for it to finish and return the data
         if self._state[datatype].active:
             try:
-                _LOGGER.debug("Already in progress. Waiting for data to be fetched")
+                _LOGGER.debug(
+                    "Already in progress. Waiting for data to be fetched"
+                )
                 await asyncio.wait_for(
-                    self._state[datatype].inactive_event.wait(), DEFAULT_TIMEOUT
+                    self._state[datatype].inactive_event.wait(),
+                    DEFAULT_TIMEOUT,
                 )
             except asyncio.TimeoutError:
-                _LOGGER.debug("Timeout while waiting for data. Will try fetching again")
+                _LOGGER.debug(
+                    "Timeout while waiting for data. Will try fetching again"
+                )
 
         # Check if we have the data already and not forcing a refresh
         if self._state[datatype].data and not force:
             # Check if the data is younger than the cache time
-            if datetime.now(timezone.utc) - self._state[datatype].timestamp < timedelta(
-                seconds=self._cache_time
-            ):
+            if datetime.now(timezone.utc) - self._state[
+                datatype
+            ].timestamp < timedelta(seconds=self._cache_time):
                 _LOGGER.debug(
                     "Using cached data for `%s`: %s",
                     datatype,
@@ -419,6 +656,9 @@ class AsusRouter:
         # Mark the data as active
         self._state[datatype].start()
 
+        # Check prerequisites
+        await self._check_prerequisites(datatype)
+
         # Get the data finder
         data_finder = self._where_to_get_data(datatype)
 
@@ -447,6 +687,14 @@ class AsusRouter:
                     )
                 # Check that we are not fetching this data already
 
+                # Add the request from kwargs
+                kw_request = kwargs.get("request", {})
+                if isinstance(kw_request, dict):
+                    for key, value in kw_request.items():
+                        request += f"{key}={value};"
+                    # Remove trailing symbol
+                    request = request[:-1]
+
                 # Fetch the data
                 data = await self.async_api_load(endpoint, request)
 
@@ -454,17 +702,24 @@ class AsusRouter:
                 if not self._identity:
                     self._identity = await self.async_get_identity()
 
-                result = merge_dicts(
-                    result,
-                    process(
-                        endpoint,
-                        data,
-                        self._state,
-                        self._identity.firmware,
-                        self._identity.wlan,
-                    ),
+                processed = process(
+                    endpoint,
+                    data,
+                    self._state,
+                    self._identity.firmware,
+                    self._identity.wlan,
                 )
 
+                # Check whether data should be dropped
+                to_drop = []
+                for key, value in processed.items():
+                    if self._drop_data(key, endpoint):
+                        to_drop.append(key)
+                for key in to_drop:
+                    processed.pop(key, None)
+
+                result = merge_dicts(result, processed)
+
                 # Check if we have data and data finder merge is ANY
                 if result and data_finder.merge == AsusDataMerge.ANY:
                     break
@@ -480,18 +735,21 @@ class AsusRouter:
                     self._state[key] = AsusDataState()
                 self._state[key].update(value)
         except (AsusRouterConnectionError, AsusRouterDataError):
-            return self._state[datatype].data
+            return self._return_state(datatype, **kwargs)
 
         # Check flags
         await self._check_flags()
 
+        # Check postrequisites
+        await self._check_postrequisites(datatype)
+
         # Return the data we were looking for
         _LOGGER.debug(
             "Returning data for `%s` with object type `%s`",
             datatype,
             type(self._state[datatype].data),
         )
-        return self._state[datatype].data
+        return self._return_state(datatype, **kwargs)
 
     # ---------------------------
     # Service-related methods -->
@@ -499,7 +757,7 @@ class AsusRouter:
 
     async def async_run_service(
         self,
-        service: str,
+        service: Optional[str],
         arguments: Optional[dict[str, Any]] = None,
         apply: bool = False,
         expect_modify: bool = True,
@@ -510,7 +768,7 @@ class AsusRouter:
         _LOGGER.debug("Triggered method async_run_service")
 
         # Run the service
-        result = await async_call_service(
+        result, self._needed_time, self._last_id = await async_call_service(
             self.async_api_command,
             service,
             arguments,
@@ -523,11 +781,42 @@ class AsusRouter:
 
         return result
 
+    async def _async_check_state_dependency(self, state: AsusState) -> None:
+        """Check and queue state dependencies. Required for some states."""
+
+        _LOGGER.debug("Triggered method _async_check_state_dependency")
+
+        dependency = get_datatype(state)
+
+        if dependency == AsusData.VPNC:
+            # VPNC state change requires the correct previous state
+            await self.async_get_data(AsusData.VPNC, force=True)
+
+        if dependency == AsusData.AURA:
+            # Aura state change requires the correct previous state
+            await self.async_get_data(AsusData.AURA, force=True)
+
+        return
+
+    async def _async_get_state_callback(
+        self, state: AsusState
+    ) -> Callable[..., Awaitable]:
+        """Get the state callback."""
+
+        _LOGGER.debug("Triggered method _async_get_state_callback")
+
+        datatype = get_datatype(state)
+        # If state is one of AsusState.AURA enum
+        if datatype == AsusData.AURA:
+            return self.async_api_command
+
+        return self.async_run_service
+
     async def async_set_state(
         self,
         state: AsusState,
-        arguments: Optional[dict[str, Any]] = None,
         expect_modify: bool = False,
+        **kwargs: Any,
     ) -> bool:
         """Set the state."""
 
@@ -536,19 +825,60 @@ class AsusRouter:
         _LOGGER.debug(
             "Setting state `%s` with arguments `%s`. Expecting modify: `%s`",
             state,
-            arguments,
+            kwargs,
             expect_modify,
         )
 
+        # Check dependencies
+        await self._async_check_state_dependency(state)
+
+        # Get the state callback
+        callback = await self._async_get_state_callback(state)
+
         result = await set_state(
-            self.async_run_service, state, arguments, expect_modify
+            callback=callback,
+            state=state,
+            expect_modify=expect_modify,
+            router_state=self._state,
+            identity=self._identity,
+            **kwargs,
         )
 
+        # Rewrite the result if it is the default one
+        if result == DEFAULT_RESULT_SUCCESS:
+            result = True
+
         if result is True:
-            # Check if we have a state object for this data
-            self._check_state(get_datatype(state))
-            # Save the state
-            save_state(state, self._state)
+            _datatype = get_datatype(state)
+
+            if _datatype in (AsusData.VPNC, AsusData.AURA):
+                # The only way to make it work with VPN Fusion
+                await asyncio.sleep(1)
+                await self._async_check_state_dependency(state)
+            elif (
+                get_enum_key_by_value(
+                    AsusState, type(state), default=AsusState.NONE
+                )
+                == AsusState.PC_RULE
+            ):
+                # We should not save this state, since it is saved differently
+                pass
+            else:
+                # Check if we have a state object for this data
+                self._check_state(_datatype)
+                # Save the state
+                _LOGGER.debug(
+                    "Saving state `%s` for `%s` s with id=`%s`",
+                    state,
+                    self._needed_time,
+                    self._last_id,
+                )
+                save_state(
+                    state, self._state, self._needed_time, self._last_id
+                )
+                # Reset the needed time and last id
+                self._needed_time = None
+                self._last_id = None
 
         return result
 
@@ -563,72 +893,6 @@ class AsusRouter:
     # This part can be changed / removed in the future
     # It might also not be working properly
 
-    async def async_apply_parental_control_rules(
-        self,
-        rules: dict[str, ParentalControlRule],
-    ) -> bool:
-        """Apply parental control rules."""
-
-        request = legacy.compile_parental_control(rules)
-
-        if request:
-            return await self.async_run_service(
-                service="restart_firewall",
-                arguments=request,
-                apply=True,
-            )
-
-        return False
-
-    async def async_remove_parental_control_rules(
-        self,
-        macs: Optional[str | list[str]] = None,
-        rules: Optional[ParentalControlRule | list[ParentalControlRule]] = None,
-        apply: bool = True,
-    ) -> dict[str, ParentalControlRule]:
-        """Remove parental control rules"""
-
-        _macs = set() if macs is None else set(safe_list(macs))
-        rules = [] if rules is None else safe_list(rules)
-
-        # Get current rules
-        current_rules: dict = (await self.async_get_data(AsusData.PARENTAL_CONTROL))[
-            "rules"
-        ]
-
-        # Remove old rules for these MACs
-        for mac in _macs:
-            current_rules.pop(mac, None)
-        for rule in rules:
-            current_rules.pop(rule.mac, None)
-
-        # Apply new rules
-        if apply:
-            await self.async_apply_parental_control_rules(current_rules)
-
-        # Return the new rules
-        return current_rules
-
-    async def async_set_parental_control_rules(
-        self,
-        rules: ParentalControlRule | list[ParentalControlRule],
-    ) -> bool:
-        """Set parental control rules"""
-
-        rules = safe_list(rules)
-
-        # Remove old rules for these MACs and get the rest of the list
-        current_rules = await self.async_remove_parental_control_rules(
-            rules, apply=False
-        )
-
-        # Add new rules
-        for rule in rules:
-            current_rules[rule.mac] = rule
-
-        # Apply new rules
-        return await self.async_apply_parental_control_rules(current_rules)
-
     async def async_apply_port_forwarding_rules(
         self,
         rules: list[PortForwardingRule],
@@ -663,7 +927,9 @@ class AsusRouter:
         )["rules"]
 
         # Remove all rules for these IPs
-        current_rules = [rule for rule in current_rules if rule.ip_address not in ips]
+        current_rules = [
+            rule for rule in current_rules if rule.ip_address not in ips
+        ]
         # Remove exact rules
         for rule_to_find in rules:
             current_rules = [
@@ -677,7 +943,10 @@ class AsusRouter:
                         rule_to_find.ip_external is None
                         or rule.ip_external == rule_to_find.ip_external
                     )
-                    and (rule_to_find.port is None or rule.port == rule_to_find.port)
+                    and (
+                        rule_to_find.port is None
+                        or rule.port == rule_to_find.port
+                    )
                 )
             ]
 
@@ -726,6 +995,14 @@ class AsusRouter:
     # <-- Properties
     # ---------------------------
 
+    # ---------------------------
+    # Additional settings -->
+    # ---------------------------
+
+    # ---------------------------
+    # <-- Additional settings
+    # ---------------------------
+
     # ---------------------------
     # General management -->
     # ---------------------------
