@@ -9,23 +9,22 @@ import asyncio
 import base64
 import json
 import logging
-from typing import Any, Optional, Tuple
+from typing import Any, Awaitable, Callable, Optional, Tuple
 from urllib.parse import quote
 
 import aiohttp
 
-from asusrouter.const import DEFAULT_TIMEOUTS, USER_AGENT
+from asusrouter.const import DEFAULT_TIMEOUTS, USER_AGENT, RequestType
 from asusrouter.error import (
     AsusRouter404Error,
     AsusRouterAccessError,
     AsusRouterConnectionError,
     AsusRouterError,
     AsusRouterLogoutError,
-    AsusRouterSessionError,
     AsusRouterTimeoutError,
 )
-from asusrouter.modules.endpoint import Endpoint, EndpointControl, EndpointService
-from asusrouter.modules.endpoint.error import AccessError, handle_access_error
+from asusrouter.modules.endpoint import EndpointService, EndpointType
+from asusrouter.modules.endpoint.error import handle_access_error
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -64,6 +63,7 @@ class Connection:  # pylint: disable=too-many-instance-attributes
         port: Optional[int] = None,
         use_ssl: bool = False,
         session: Optional[aiohttp.ClientSession] = None,
+        dumpback: Optional[Callable[..., Awaitable[None]]] = None,
     ):
         """Initialize connection."""
 
@@ -75,7 +75,7 @@ class Connection:  # pylint: disable=too-many-instance-attributes
         self._password = password
 
         # Client session
-        self._session = session if session is not None else aiohttp.ClientSession()
+        self._session = session if session is not None else self._new_session()
         _LOGGER.debug("Using session `%s`", session)
 
         # Set the port and protocol based on the input
@@ -83,6 +83,14 @@ class Connection:  # pylint: disable=too-many-instance-attributes
         self._port = port or (8443 if use_ssl else 80)
         _LOGGER.debug("Using `%s` and port `%s`", self._http, self._port)
 
+        # Callback for dumping data
+        self._dumpback = dumpback
+
+    def _new_session(self) -> aiohttp.ClientSession:
+        """Create a new session."""
+
+        return aiohttp.ClientSession()
+
     async def async_connect(
         self, retry: int = 0, lock: Optional[asyncio.Lock] = None
     ) -> bool:
@@ -114,19 +122,11 @@ class Connection:  # pylint: disable=too-many-instance-attributes
                 )
                 _LOGGER.debug("Received authorization response")
             except AsusRouterAccessError as ex:
-                # Check the AccessError
-                if ex.args[1] == AccessError.TRY_AGAIN:
-                    # Get the timeout
-                    timeout = ex.args[2].get("timeout") + 5
-                    if timeout:
-                        _LOGGER.debug("Will try again in %s seconds", timeout)
-                        await asyncio.sleep(timeout)
-                        return await self.async_connect(retry, asyncio.Lock())
-                    raise ex
-                # In case of any other error, switch to the next except block
-                raise ex
+                raise AsusRouterAccessError(
+                    f"Cannot access {EndpointService.LOGIN}. Failed in `async_connect`"
+                ) from ex
             except AsusRouterError as ex:
-                _LOGGER.debug("Connection failed")
+                _LOGGER.debug("Connection failed with error: %s", ex)
                 if retry < len(DEFAULT_TIMEOUTS) - 1:
                     _LOGGER.debug(
                         "Will try again in %s seconds", DEFAULT_TIMEOUTS[retry]
@@ -134,8 +134,9 @@ class Connection:  # pylint: disable=too-many-instance-attributes
                     await asyncio.sleep(DEFAULT_TIMEOUTS[retry])
                     return await self.async_connect(retry + 1, asyncio.Lock())
                 raise AsusRouterTimeoutError(
-                    f"Reached maximum allowed timeout\
-                        for a single connection attempt: {sum(DEFAULT_TIMEOUTS)}"
+                    f"Reached maximum allowed timeout \
+for a single connection attempt: {sum(DEFAULT_TIMEOUTS)}. \
+The last error: {ex}"
                 ) from ex
             except Exception as ex:  # pylint: disable=broad-except
                 _LOGGER.debug("Error while connecting to %s: %s", self._hostname, ex)
@@ -188,18 +189,20 @@ class Connection:  # pylint: disable=too-many-instance-attributes
 
     async def _send_request(
         self,
-        endpoint: Endpoint | EndpointControl | EndpointService,
+        endpoint: EndpointType,
         payload: Optional[str] = None,
         headers: Optional[dict[str, str]] = None,
+        request_type: RequestType = RequestType.POST,
     ) -> Tuple[int, dict[str, str], str]:
         """Send a request to the device."""
 
         # Send request
         try:
-            resp_status, resp_headers, resp_content = await self._make_post_request(
+            resp_status, resp_headers, resp_content = await self._make_request(
                 endpoint,
                 payload,
                 headers,
+                request_type,
             )
 
             # Raise exception on 404
@@ -221,17 +224,24 @@ class Connection:  # pylint: disable=too-many-instance-attributes
             return (resp_status, resp_headers, resp_content)
         except aiohttp.ClientConnectorError as ex:
             self.reset_connection()
-            raise AsusRouterConnectionError from ex
+            raise AsusRouterConnectionError(
+                f"Cannot connect to `{self._hostname}` on port `{self._port}`. \
+Failed in `_send_request` with error: `{ex}`"
+            ) from ex
         except (aiohttp.ClientConnectionError, aiohttp.ClientOSError) as ex:
-            raise AsusRouterConnectionError from ex
+            raise AsusRouterConnectionError(
+                f"Cannot connect to `{self._hostname}` on port `{self._port}`. \
+Failed in `_send_request` with error: `{ex}`"
+            ) from ex
         except Exception as ex:  # pylint: disable=broad-except
             raise ex
 
     async def async_query(
         self,
-        endpoint: Endpoint | EndpointControl | EndpointService,
+        endpoint: EndpointType,
         payload: Optional[str] = None,
         headers: Optional[dict[str, str]] = None,
+        request_type: RequestType = RequestType.POST,
     ) -> Tuple[int, dict[str, str], str]:
         """Send a request to the device."""
 
@@ -245,22 +255,29 @@ class Connection:  # pylint: disable=too-many-instance-attributes
             raise AsusRouterTimeoutError("Data cannot be retrieved. Connection failed")
 
         # Send the request
-        _LOGGER.debug("Sending request to %s", self._hostname)
-        return await self._send_request(endpoint, payload, headers)
+        _LOGGER.debug("Sending `%s` request to `%s`", request_type, self._hostname)
+        return await self._send_request(endpoint, payload, headers, request_type)
 
-    async def _make_post_request(
+    async def _make_request(
         self,
-        endpoint: Endpoint | EndpointControl | EndpointService,
+        endpoint: EndpointType,
         payload: Optional[str] = None,
         headers: Optional[dict[str, str]] = None,
+        request_type: RequestType = RequestType.POST,
     ) -> Any:
         """Make a post request to the device."""
 
         # Check if a session is available
-        if self._session is None:
+        if self._session is None or self._session.closed:
             # If no session is available, we cannot be connected to the device
-            # So we don't try to make any requests
-            raise AsusRouterSessionError("No session available for this connection")
+            _LOGGER.debug("No session available. Creating a new one")
+            # We will create a new session and retry the request
+            self.reset_connection()
+            self._session = self._new_session()
+            # Reconnect
+            await self.async_connect()
+            # Retry the request
+            return await self._make_request(endpoint, payload, headers, request_type)
 
         # Check headers
         if not headers:
@@ -269,13 +286,19 @@ class Connection:  # pylint: disable=too-many-instance-attributes
         # Generate the url
         url = f"{self._http}://{self._hostname}:{self._port}/{endpoint.value}"
 
+        # Add get parameters if needed
+        if request_type == RequestType.GET and payload:
+            payload.replace(";", "&")
+            url = f"{url}?{payload}"
+
         # Process the payload to be sent
         payload_to_send = quote(payload) if payload else None
 
         # Send the request
-        async with self._session.post(
+        async with self._session.request(
+            request_type.value,
             url,
-            data=payload_to_send,
+            data=payload_to_send if request_type == RequestType.POST else None,
             headers=headers,
             ssl=self._ssl,
         ) as response:
@@ -292,6 +315,12 @@ class Connection:  # pylint: disable=too-many-instance-attributes
                 _LOGGER.debug("Cannot decode response. Will ignore errors")
                 resp_content = await response.text(errors="ignore")
 
+            # Call the dumpback if available
+            if self._dumpback is not None:
+                await self._dumpback(
+                    endpoint, payload, resp_status, resp_headers, resp_content
+                )
+
             # Return the response
             return (resp_status, resp_headers, resp_content)
 
