@@ -4,14 +4,15 @@ from __future__ import annotations
 
 import importlib
 import logging
+from concurrent.futures import ThreadPoolExecutor
 from enum import Enum
 from types import ModuleType
-from typing import Any, Awaitable, Callable, Optional
+from typing import Any, Awaitable, Callable, Optional, Union
 
+from asusrouter.const import RequestType
 from asusrouter.error import AsusRouter404Error
 from asusrouter.modules.data import AsusData, AsusDataState
 from asusrouter.modules.firmware import Firmware
-from asusrouter.modules.flags import Flag
 from asusrouter.modules.wlan import Wlan
 
 _LOGGER = logging.getLogger(__name__)
@@ -23,16 +24,19 @@ class Endpoint(str, Enum):
     DEVICEMAP = "ajax_status.xml"
     ETHERNET_PORTS = "ajax_ethernet_ports.asp"
     FIRMWARE = "detect_firmware.asp"
+    FIRMWARE_NOTE = "release_note0.asp"
+    FIRMWARE_NOTE_AIMESH = "release_note_amas.asp"
     HOOK = "appGet.cgi"
     NETWORKMAPD = "update_networkmapd.asp"
     ONBOARDING = "ajax_onboarding.asp"
     PORT_STATUS = "get_port_status.cgi"
-    RGB = "light_effect.html"
     STATE = "state.js"
     SYSINFO = "ajax_sysinfo.asp"
     TEMPERATURE = "ajax_coretmp.asp"
     UPDATE_CLIENTS = "update_clients.asp"
     VPN = "ajax_vpn_status.asp"
+    # Endpoints known but not used / not available on test devices
+    # RGB = "light_effect.html"
 
 
 class EndpointControl(str, Enum):
@@ -40,7 +44,6 @@ class EndpointControl(str, Enum):
 
     APPLY = "apply.cgi"
     COMMAND = "applyapp.cgi"
-    LEDG = "set_ledg.cgi"
 
 
 class EndpointService(str, Enum):
@@ -50,14 +53,45 @@ class EndpointService(str, Enum):
     LOGOUT = "Logout.asp"
 
 
-def _get_module(endpoint: Endpoint | EndpointControl) -> Optional[ModuleType]:
+class EndpointTools(str, Enum):
+    """Tools endpoints."""
+
+    # AURA control / RGB
+    AURA = "set_ledg.cgi"
+    # Network tools: ping, jitter, loss
+    NETWORK = "netool.cgi"
+
+
+class EndpointNoCheck(str, Enum):
+    """Endpoints that should not be checked for availability."""
+
+    # AURA control / RGB
+    AURA = "set_ledg.cgi"
+
+
+# Typehint for the endpoint
+EndpointType = Union[Endpoint, EndpointControl, EndpointService, EndpointTools]
+
+# Force request type for the endpoint
+ENDPOINT_FORCE_REQUEST = {
+    EndpointTools.NETWORK: RequestType.GET,
+}
+
+
+def _get_module(
+    endpoint: Endpoint | EndpointControl | EndpointTools,
+) -> Optional[ModuleType]:
     """Attempt to get the module for the endpoint."""
 
     try:
         # Get the module name from the endpoint
         module_name = f"asusrouter.modules.endpoint.{endpoint.name.lower()}"
-        # Import the module
-        submodule = importlib.import_module(module_name)
+
+        # Import the module in a separate thread to avoid blocking the main thread
+        with ThreadPoolExecutor(max_workers=1) as executor:
+            future = executor.submit(importlib.import_module, module_name)
+            submodule = future.result()
+
         # Return the module
         return submodule
     except ModuleNotFoundError:
@@ -65,7 +99,9 @@ def _get_module(endpoint: Endpoint | EndpointControl) -> Optional[ModuleType]:
         return None
 
 
-def read(endpoint: Endpoint | EndpointControl, content: str) -> dict[str, Any]:
+def read(
+    endpoint: Endpoint | EndpointControl | EndpointTools, content: str
+) -> dict[str, Any]:
     """Read the data from an endpoint."""
 
     _LOGGER.debug("Reading data from endpoint %s", endpoint)
@@ -126,7 +162,7 @@ def data_set(data: dict[str, Any], **kwargs: Any) -> dict[str, Any]:
     return data
 
 
-def data_get(data: dict[str, Any], key: str) -> Any:
+def data_get(data: dict[str, Any], key: str) -> Optional[Any]:
     """Extract value from the data dict and update the data dict."""
 
     # Get the value
@@ -140,15 +176,16 @@ def data_get(data: dict[str, Any], key: str) -> Any:
 
 
 async def check_available(
-    endpoint: Endpoint, api_query: Callable[..., Awaitable[Any]]
-) -> bool:
+    endpoint: Endpoint | EndpointTools,
+    api_query: Callable[..., Awaitable[Any]],
+) -> tuple[bool, Optional[Any]]:
     """Check whether the endpoint is available or returns 404."""
 
     try:
-        status, _, _ = await api_query(endpoint)
+        status, _, content = await api_query(endpoint)
         if status == 200:
-            return True
+            return (True, content)
     except AsusRouter404Error:
-        return False
+        return (False, None)
 
-    return False
+    return (False, None)
