@@ -6,11 +6,25 @@ from __future__ import annotations
 
 import logging
 from dataclasses import dataclass
+from itertools import chain
 from typing import Any, Awaitable, Callable, Optional, Tuple
 
 from asusrouter.error import AsusRouterIdentityError
-from asusrouter.modules.endpoint import Endpoint, check_available
-from asusrouter.modules.firmware import Firmware, read_fw_string
+from asusrouter.modules.aimesh import AiMeshDevice
+from asusrouter.modules.color import color_zone
+from asusrouter.modules.data import AsusData
+from asusrouter.modules.endpoint import (
+    Endpoint,
+    EndpointNoCheck,
+    EndpointTools,
+    EndpointType,
+    check_available,
+)
+from asusrouter.modules.endpoint.onboarding import (
+    process as process_onboarding,
+)
+from asusrouter.modules.endpoint.onboarding import read as read_onboarding
+from asusrouter.modules.firmware import Firmware
 from asusrouter.modules.wlan import WLAN_TYPE, Wlan
 from asusrouter.tools import writers
 from asusrouter.tools.cleaners import clean_dict
@@ -27,6 +41,7 @@ MAP_IDENTITY: Tuple = (
     ("label_mac", "mac"),
     ("lan_hwaddr", "lan_mac"),
     ("wan_hwaddr", "wan_mac"),
+    ("productid", "product_id"),
     ("productid", "model"),
     ("firmver", "fw_major"),
     ("buildno", "fw_minor"),
@@ -34,6 +49,8 @@ MAP_IDENTITY: Tuple = (
     ("rc_support", "services", safe_list_from_string),
     ("ss_support", "services", safe_list_from_string),
     ("led_val", "led", safe_exists),
+    ("ledg_rgb1", "aura", safe_exists),
+    ("ledg_rgb2", "aura_zone", color_zone),
 )
 
 
@@ -47,19 +64,25 @@ class AsusDevice:  # pylint: disable=too-many-instance-attributes
     # Device-defining values
     serial: Optional[str] = None
     mac: Optional[str] = None
+    product_id: Optional[str] = None
     model: Optional[str] = None
     brand: str = "ASUSTek"
 
+    # Supported features
+    aimesh: bool = False
+
     # Device information
     firmware: Optional[Firmware] = None
+    merlin: bool = False
     wlan: Optional[list[Wlan]] = None
-    endpoints: Optional[dict[Endpoint, bool]] = None
+    endpoints: Optional[dict[EndpointType, bool]] = None
     services: Optional[list[str]] = None
 
     # Flags for device features
-    led: bool = False
-    ledg: bool = False
     aura: bool = False
+    aura_zone: int = 0
+    led: bool = False
+    ookla: bool = False
     vpn_status: bool = False
 
 
@@ -82,18 +105,33 @@ async def collect_identity(
     try:
         identity_map = await api_hook(request)
     except Exception as ex:  # pylint: disable=broad-except
-        raise AsusRouterIdentityError from ex
+        raise AsusRouterIdentityError(
+            "Failed to collect identity data from the router"
+        ) from ex
     _LOGGER.debug("Identity collected")
 
     # Read the identity
     identity = _read_nvram(identity_map)
     _LOGGER.debug("Identity read")
 
+    # Process services
+    identity["aimesh"] = "amas" in identity["services"]
+
     # Check endpoints
-    endpoints = await _check_endpoints(api_query)
+    endpoints, onboarding = await _check_endpoints(api_query)
     identity["endpoints"] = endpoints
     _LOGGER.debug("Endpoints checked")
 
+    # Check onboarding to get nice model name
+    this_device = onboarding.get(identity["mac"])
+    if isinstance(this_device, AiMeshDevice):
+        identity["model"] = this_device.model
+
+    # Check if Merlin
+    if endpoints[Endpoint.SYSINFO] is True:
+        identity["merlin"] = True
+        _LOGGER.debug("Merlin FW detected")
+
     # Return the identity convered from a dict
     return AsusDevice(**identity)
 
@@ -104,7 +142,7 @@ def _read_nvram(data: dict[str, Any]) -> dict[str, Any]:
 
     # Check the input data
     if not data:
-        raise AsusRouterIdentityError("No data received")
+        raise AsusRouterIdentityError("No nvram data received")
 
     # Create the identity dictionary
     identity: dict[str, Any] = {}
@@ -139,7 +177,7 @@ def _read_nvram(data: dict[str, Any]) -> dict[str, Any]:
 
     # Firmware
     # TODO: Optimize this
-    firmware = read_fw_string(
+    firmware = Firmware(
         f"{identity['fw_major']}.{identity['fw_minor']}.{identity['fw_build']}"
     )
     identity["firmware"] = firmware
@@ -153,17 +191,32 @@ def _read_nvram(data: dict[str, Any]) -> dict[str, Any]:
         if value in WLAN_TYPE:
             identity["wlan"].append(WLAN_TYPE[value])
 
+    # OOKLA Speedtest
+    if "ookla" in identity["services"]:
+        identity["ookla"] = True
+
     return identity
 
 
 async def _check_endpoints(
-    api_hook: Callable[..., Awaitable[Any]]
-) -> dict[Endpoint, bool]:
+    api_hook: Callable[..., Awaitable[Any]],
+) -> tuple[dict[EndpointType, bool], dict[str, Any]]:
     """Check which endpoints are available."""
 
-    endpoints: dict[Endpoint, bool] = {}
+    endpoints: dict[EndpointType, bool] = {}
+    contents: dict[EndpointType, Any] = {}
 
-    for endpoint in Endpoint:
-        endpoints[endpoint] = await check_available(endpoint, api_hook)
+    for endpoint in chain(Endpoint, EndpointTools):
+        if endpoint.name in EndpointNoCheck.__members__:
+            continue
+        result, content = await check_available(endpoint, api_hook)
+        endpoints[endpoint] = result
+        contents[endpoint] = content
 
-    return endpoints
+    onboarding = {}
+    if endpoints[Endpoint.ONBOARDING] is True:
+        onboarding = process_onboarding(
+            read_onboarding(contents[Endpoint.ONBOARDING])
+        ).get(AsusData.AIMESH, {})
+
+    return (endpoints, onboarding)
